@@ -7,6 +7,7 @@ import uuid
 import time
 import re
 import socket
+import traceback
 
 import paramiko
 
@@ -75,11 +76,14 @@ class SshShell(object):
         stderr = kwargs.pop("stderr", None)
         allow_error = kwargs.pop("allow_error", False)
         store_pid = kwargs.pop("store_pid", False)
+        use_pty = kwargs.pop("use_pty", False)
         command_in_cwd = self._generate_run_command(command, *args, store_pid=store_pid, **kwargs)
         try:
             channel = self._get_ssh_transport().open_session()
         except EOFError as error:
             raise self._connection_error(error)
+        if use_pty:
+            channel.get_pty()
         channel.exec_command(command_in_cwd)
         
         process_stdout = channel.makefile('rb')
@@ -205,10 +209,13 @@ class SshShell(object):
         return self._get_ssh_transport().open_sftp_client()
         
     def _connection_error(self, error):
-        return ConnectionError(
+        connection_error = ConnectionError(
             "Error creating SSH connection\n" +
             "Original error: {0}".format(error)
         )
+        connection_error.original_error = error
+        connection_error.original_traceback = traceback.format_exc()
+        return connection_error
 
 
 class SftpFile(object):
@@ -251,7 +258,7 @@ class SshProcess(object):
         self._io = IoHandler([
             (self._stdout, stdout),
             (self._stderr, stderr),
-        ], lambda: (self._stdout.read(), self._stderr.read()))
+        ])
         
     def is_running(self):
         return not self._channel.exit_status_ready()
